#include "ac33mx128_pcu.h"
#include "ac33mx128_scu.h"
#include "ac33mx128_dmac.h"
#include "ac33mx128_adc.h"
#include "debug_frmwrk.h"
#include "ac33mx128_libcfg.h"

/*
CHSEL    ADC0                ADC1               ADC2
--------------------------------------------------------
0000        AIN0	               AIN0                 AIN0                 CH0
0001        AIN0_OPAMP   AIN0_OPAMP    AIN0_OPAMP    CH1
0010        AIN1                 AIN1                 AIN1                 CH2
0011        AIN1_OPAMP   AIN1_OPAMP    AIN1_OPAMP    CH3
0100        AIN2                 AIN2                 AIN2                 CH4
0101        AIN2_OPAMP   AIN2_OPAMP    AIN2_OPAMP    CH5
0110        AIN3                 AIN3                 AIN3                 CH6
0111        AIN3_OPAMP   AIN3_OPAMP    AIN3_OPAMP    CH7
1000        AIN4                 AIN4                 AIN4                 CH8
1001        AIN5                 AIN5                 AIN5                 CH9
1010        AIN6                 AIN6                 AIN6                 CH10
1011        AIN7                 AIN10               AIN13                CH11
1100        AIN8                 AIN11               AIN14                CH12
1101        AIN9                 AIN12               AIN15                CH13
1110        -                      -                      -                      CH14
1111        -                      -                      -                      CH15
---------------------------------------------------------
*/

// system default I/O setting
void PCU_Init(void);

const uint8_t menu[] =
"************************************************\n\r"
" ADC demo \n\r"
"\t - MCU: AC33M8128 \n\r"
"\t - Core: ARM Cortex-M3 \n\r"
"\t - Communicate via: UART0 - 115200 bps \n\r"
" Using ADC2[AN13,AN14,AN15] in interrupt mode \n\r"
"************************************************\n\r";
/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void print_menu(void)
{
	_DBG(menu);
}

uint32_t fflag;

void ADC2_IRQHandler(void)
{
	uint16_t status;
	status = ADC_GetStatus(AD2);

	if (status & ADC_STAT_SINGLE){
		fflag=1;
		ADC_ClearStatus(AD2, status);	
	}
}

uint16_t adcaverage(uint16_t *val)
{
	uint32_t i;
	uint32_t adcavg;
	
	adcavg=0;			
	for (i=4;i<8;i++){	
		adcavg+=(val[i]>>4);
	}
	adcavg>>=2;
	
	return 	(adcavg);
}

uint16_t adcval[8];

void mainloop(void)
{
	uint32_t i;
	uint32_t adcavg;
	uint32_t tmp;
	ADC_CFG_Type AD2_config;
	
	print_menu();

	AD2_config.Mode = ADC_SINGLE_MODE;
	AD2_config.UseClk = ADC_INTERNAL_CLK;
	AD2_config.SampingTime=0;
	AD2_config.InClkDiv = (((SystemPeriClock/1000000UL) * 2)/ (3*(15+AD2_config.SampingTime))) + 1; 
	ADC_Init(AD2, &AD2_config);
	ADC_IntConfig(AD2, ADC_INTEN_SINGLE, ENABLE);

	NVIC_SetPriority(ADC2_IRQn, ((0x01<<1)|0x01));
	/* Enable Interrupt for ADC2 channel */
    NVIC_EnableIRQ(ADC2_IRQn);
	
	__enable_irq();
	
	_DBG_(" ");
	while(1){
		ADC_ChannelSel(AD2, 11);	
		for (i=0;i<16;i++){
			fflag=0;
			ADC_StartCmd(AD2); // start 	
			while(fflag==0){}
			adcval[i]=ADC_GetData(AD2);
		}
		adcavg=adcaverage(adcval);
		_DBG("AN13 = "); _DBH16(adcavg); _DBG(" ");		
		
		
		ADC_ChannelSel(AD2, 12);			
		for (i=0;i<16;i++){
			fflag=0;
			ADC_StartCmd(AD2); // start 	
			while(fflag==0){}
			adcval[i]=ADC_GetData(AD2);
		}
		adcavg=adcaverage(adcval);
		_DBG("AN14 = "); _DBH16(adcavg); _DBG(" ");				
		
		
		ADC_ChannelSel(AD2, 13);				
		for (i=0;i<16;i++){
			fflag=0;
			ADC_StartCmd(AD2); // start 	
			while(fflag==0){}
			adcval[i]=ADC_GetData(AD2);
		}		
		adcavg=adcaverage(adcval);
		_DBG("AN15 = "); _DBH16(adcavg); _DBG_(" ");
		
		for(tmp = 0; tmp < 1000000; tmp++);
	}
}


int main(void)
{
	SystemInit();
	PCU_Init(); // all port control setting  !!You have to set PORT function!!
	SCU_ClockInit(); // MCLK=72Mhz(PLL) setting 
	debug_frmwrk_init();
	
	mainloop();
	
	return (0);
}


void PCU_Init(void)
{
//Peripheral Enable Register 1  0:Disable, 1:Enable	
	SCU->PER1=SCU->PER1  
	| (1<<11)   // GPIOD
	| (1<<10)   // GPIOC
	| (1<<9)   // GPIOB
	| (1<<8)   // GPIOA
		; 		
//Peripheral Clock Enable Register 1 0:Disable, 1:Enable	
	SCU->PCER1=SCU->PCER1
	| (1<<11)   // GPIOD
	| (1<<10)   // GPIOC
	| (1<<9)   // GPIOB
	| (1<<8)   // GPIOA
		; 	
	
 	PORT_ACCESS_EN();  // enable writing permittion of ALL PCU register

	PCA->MR = 0 // PinMux setting (00b,01b,10b,11b)
	| (3UL<<30)   // 0:PA15, 1:MISO0, 2:T7C, 3:AN15
	| (3UL<<28)   // 0:PA14, 1:MOSI0,            3:AN14
	| (3UL<<26)   // 0:PA13, 1:SCK0,              3:AN13
	| (0UL<<24)   // 0:PA12, 1:SS0, 2:AD2I,     3:AN12
	| (0UL<<22)   // 0:PA11, 1:TRACED2,        3:AN11
	| (0UL<<20)   // 0:PA10, 1:TRACED1, 2:AD2O, 3:AN10
	| (0UL<<18)   // 0:PA9,  1:TRACED0, 2:AD1O, 3:AN9
	| (0UL<<16)   // 0:PA8, 1:TRACECLK, 2:AD0O, 3:AN8
	| (0UL<<14)   // 0:PA7, 1:TRACED3, 2:T3O, 3:AN7_CREF1
	| (0UL<<12)   // 0:PA6,                     2:T2O, 3:AN6_CREF0
	| (0UL<<10)   // 0:PA5,                     2:T1O, 3:AN5
	| (0UL<<8)   // 0:PA4,                       2:T0O, 3:AN4
	| (0UL<<6)   // 0:PA3,                            3:AN3_CP3
	| (0UL<<4)   // 0:PA2,                            3:AN2_CP2	
	| (0UL<<2)   // 0:PA1,                            3:AN1_CP1	
	| (0UL<<0)   // 0:PA0,                            3:AN0_CP0		
		; 
	PCA->CR = 0 // 0xFFFFFFFF(reset value) Input, output, open drain, analog
	| (3UL<<30)   // P15 0:Push-pull output, 1:Open-drain output, 2:Input, 3:Analog
	| (3UL<<28)   // P14
	| (3UL<<26)   // P13
	| (0UL<<24)   // P12
	| (0UL<<22)   // P11
	| (0UL<<20)   // P10
	| (0UL<<18)   // P9
	| (0UL<<16)   // P8
	| (0UL<<14)   // P7
	| (0UL<<12)   // P6
	| (0UL<<10)   // P5
	| (0UL<<8)   // P4
	| (0UL<<6)   // P3
	| (0UL<<4)   // P2
	| (0UL<<2)   // P1
	| (0UL<<0)   // P0	
		; 	
	PCA->PCR = 0 // pullup (0,1)
	| (0<<15)   // P15 0:Disable pull-up resistor, 1:Enable pull-up resister
	| (0<<14)   // P14
	| (0<<13)   // P13
	| (0<<12)   // P12
	| (0<<11)   // P11
	| (0<<10)   // P10
	| (0<<9)   // P9
	| (0<<8)   // P8
	| (0<<7)   // P7
	| (0<<6)   // P6
	| (0<<5)   // P5
	| (0<<4)   // P4
	| (0<<3)   // P3
	| (0<<2)   // P2
	| (0<<1)   // P1
	| (0<<0)   // P0	
		; 		
	
	PCB->MR = 0 // PinMux setting (00b,01b,10b,11b)
	| (0UL<<30)   // 0:PB15, 1:MP1WL
	| (0UL<<28)   // 0:PB14, 1:MP1WH
	| (0UL<<26)   // 0:PB13, 1:MP1VL
	| (0UL<<24)   // 0:PB12, 1:MP1VH
	| (0UL<<22)   // 0:PB11, 1:MP1UL
	| (0UL<<20)   // 0:PB10, 1:MP1UH
	| (0UL<<18)   // 0:PB9,  1:OVIN1, 2:TXD3
	| (0UL<<16)   // 0:PB8, 1:PRTIN1, 2:RXD3
	| (0UL<<14)   // 0:PB7, 1:OVIN0, 2:STBYO
	| (0UL<<12)   // 0:PB6, 1:PRTIN0, 2:WDTO
	| (0UL<<10)   // 0:PB5, 1:MP0WL, 2:T9O
	| (0UL<<8)   // 0:PB4, 1:MP0WH, 2:T9C
	| (0UL<<6)   // 0:PB3, 1:MP0VL
	| (0UL<<4)   // 0:PB2, 1:MP0VH
	| (0UL<<2)   // 0:PB1, 1:MP0UL
	| (0UL<<0)   // 0:PB0, 1:MP0UH 
		;	
	PCB->CR = 0   // 0xFFFFFFFF(reset value) Input, output, open drain, analog
	| (0UL<<30)   // P15 0:Push-pull output, 1:Open-drain output, 2:Input, 3:Analog
	| (0UL<<28)   // P14
	| (0UL<<26)   // P13
	| (0UL<<24)   // P12
	| (0UL<<22)   // P11
	| (0UL<<20)   // P10
	| (0UL<<18)   // P9
	| (0UL<<16)   // P8
	| (0UL<<14)   // P7
	| (0UL<<12)   // P6
	| (0UL<<10)   // P5
	| (0UL<<8)   // P4
	| (0UL<<6)   // P3
	| (0UL<<4)   // P2
	| (0UL<<2)   // P1
	| (0UL<<0)   // P0	
		; 	
	PCB->PCR = 0 // pullup (0,1)
	| (0<<15)   // P15 0:Disable pull-up resistor, 1:Enable pull-up resister
	| (0<<14)   // P14
	| (0<<13)   // P13
	| (0<<12)   // P12
	| (0<<11)   // P11
	| (0<<10)   // P10
	| (0<<9)   // P9
	| (0<<8)   // P8
	| (0<<7)   // P7
	| (0<<6)   // P6
	| (0<<5)   // P5
	| (0<<4)   // P4
	| (0<<3)   // P3
	| (0<<2)   // P2
	| (0<<1)   // P1
	| (0<<0)   // P0	
		; 
	
	PCC->MR = 0 //0x00100155(reset value) // PinMux setting (00b,01b,10b,11b)
	| (1UL<<30)   // 0:PC15, 1:TXD0, 2:MOSI0
	| (1UL<<28)   // 0:PC14, 1:RXD0, 2:MISO0, 3:VMARGIN
	| (1UL<<26)   // 0:PC13, 1:XOUT
	| (1UL<<24)   // 0:PC12, 1:XIN
	| (0UL<<22)   // 0:PC11,            2:T8C
	| (1UL<<20)   // 0:PC10, 1:nRESET,
	| (0UL<<18)   // 0:PC9,  1:CLKO, 2:P8O
	| (0UL<<16)   // 0:PC8, 1:SDA0
	| (0UL<<14)   // 0:PC7, 1:SCL0, 2:T3C
	| (0UL<<12)   // 0:PC6, 1:TXD1, 2:T2C_PHZ
	| (0UL<<10)   // 0:PC5, 1:RXD1, 2:T1C_PHB
	| (1UL<<8)   // 0:PC4, 1:nTRST, 2:T0C_PHA
	| (1UL<<6)   // 0:PC3, 1:TDI
	| (1UL<<4)   // 0:PC2, 1:TDO_SWO
	| (1UL<<2)   // 0:PC1, 1:TMS_SWDIO
	| (1UL<<0)   // 0:PC0, 1:TCK_SWCLK 
		;		
	PCC->CR = 0   // 0xFFAFFE8A(reset value) Input, output, open drain, analog
	| (0UL<<30)   // P15 0:Push-pull output, 1:Open-drain output, 2:Input, 3:Analog
	| (2UL<<28)   // P14
	| (3UL<<26)   // P13  if XOUT, it must be Analog 
	| (3UL<<24)   // P12  if XIN, it must be Analog
	| (0UL<<22)   // P11
	| (2UL<<20)   // P10
	| (0UL<<18)   // P9
	| (0UL<<16)   // P8
	| (0UL<<14)   // P7
	| (0UL<<12)   // P6
	| (0UL<<10)   // P5
	| (2UL<<8)   // P4
	| (2UL<<6)   // P3
	| (2UL<<4)   // P2
	| (2UL<<2)   // P1
	| (2UL<<0)   // P0	
		; 	
	PCC->PCR = 0 // pullup (0,1)
	| (0<<15)   // P15 0:Disable pull-up resistor, 1:Enable pull-up resister
	| (0<<14)   // P14
	| (0<<13)   // P13
	| (0<<12)   // P12
	| (0<<11)   // P11
	| (0<<10)   // P10
	| (0<<9)   // P9
	| (0<<8)   // P8
	| (0<<7)   // P7
	| (0<<6)   // P6
	| (0<<5)   // P5
	| (0<<4)   // P4
	| (0<<3)   // P3
	| (0<<2)   // P2
	| (0<<1)   // P1
	| (0<<0)   // P0	
		; 

	PCD->MR = 0 // PinMux setting (00b,01b,10b,11b)
	| (0UL<<30)   // 0:PD15, 1:AD2EOC
	| (0UL<<28)   // 0:PD14, 1:AD2SOC
	| (0UL<<26)   // 0:PD13, 1:AD1EOC, 2:T3C
	| (0UL<<24)   // 0:PD12, 1:AD1SOC, 2:T2C_PHZ
	| (0UL<<22)   // 0:PD11, 1:AD0EOC, 2:T1C_PHB
	| (0UL<<20)   // 0:PD10, 1:AD0SOC, 2:T0C_PHA
	| (0UL<<18)   // 0:PD9,                     2:STBYO
	| (0UL<<16)   // 0:PD8,                     2:WDTO
	| (0UL<<14)   // 0:PD7, 1:RXD2, 2:AD1I
	| (0UL<<12)   // 0:PD6, 1:TXD2, 2:AD0I
	| (0UL<<10)   // 0:PD5, 1:SDA1
	| (0UL<<8)   // 0:PD4, 1:SCL1
	| (0UL<<6)   // 0:PD3, 1:MISO1
	| (0UL<<4)   // 0:PD2, 1:MOSI1
	| (0UL<<2)   // 0:PD1, 1:SCK1
	| (0UL<<0)   // 0:PD0, 1:SS1
		;		
	PCD->CR = 0   // 0xFFFFFFFF(reset value) Input, output, open drain, analog
	| (0UL<<30)   // P15 0:Push-pull output, 1:Open-drain output, 2:Input, 3:Analog
	| (0UL<<28)   // P14
	| (0UL<<26)   // P13
	| (0UL<<24)   // P12
	| (0UL<<22)   // P11
	| (0UL<<20)   // P10
	| (0UL<<18)   // P9
	| (0UL<<16)   // P8
	| (0UL<<14)   // P7
	| (0UL<<12)   // P6
	| (0UL<<10)   // P5
	| (0UL<<8)   // P4
	| (0UL<<6)   // P3
	| (0UL<<4)   // P2
	| (0UL<<2)   // P1
	| (0UL<<0)   // P0	
		; 	
	PCD->PCR = 0 // pullup (0,1)
	| (0<<15)   // P15 0:Disable pull-up resistor, 1:Enable pull-up resister
	| (0<<14)   // P14
	| (0<<13)   // P13
	| (0<<12)   // P12
	| (0<<11)   // P11
	| (0<<10)   // P10
	| (0<<9)   // P9
	| (0<<8)   // P8
	| (0<<7)   // P7
	| (0<<6)   // P6
	| (0<<5)   // P5
	| (0<<4)   // P4
	| (0<<3)   // P3
	| (0<<2)   // P2
	| (0<<1)   // P1
	| (0<<0)   // P0	
		; 
	PORT_ACCESS_DIS();  // disable writing permittion of ALL PCU register 	
}
